@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of HyperKitty.
 #
@@ -26,16 +26,17 @@ import re
 import urllib
 from email.message import EmailMessage
 
+from django.contrib.auth.models import User
+from django.test import override_settings
+
 from bs4 import BeautifulSoup
+from django_mailman3.tests.utils import get_flash_messages
 from mock import patch
 
-from django.contrib.auth.models import User
-from hyperkitty.utils import reverse
-from django_mailman3.tests.utils import get_flash_messages
-
 from hyperkitty.lib.incoming import add_to_list
-from hyperkitty.models import MailingList, Thread, Tag, Tagging, Email
-from hyperkitty.tests.utils import TestCase, SearchEnabledTestCase
+from hyperkitty.models import Email, MailingList, Tag, Tagging, Thread
+from hyperkitty.tests.utils import SearchEnabledTestCase, TestCase
+from hyperkitty.utils import reverse
 
 
 class ReattachTestCase(SearchEnabledTestCase):
@@ -161,7 +162,7 @@ class ThreadTestCase(TestCase):
         msg = self._make_msg("msgid")
         self.threadid = msg["Message-ID-Hash"]
 
-    def _make_msg(self, msgid, headers=None):
+    def _make_msg(self, msgid, headers=None, listname="list@example.com"):
         if headers is None:
             headers = {}
         msg = EmailMessage()
@@ -174,7 +175,7 @@ class ThreadTestCase(TestCase):
                 msg.replace_header(header, value)
             else:
                 msg[header] = value
-        msg["Message-ID-Hash"] = add_to_list("list@example.com", msg)
+        msg["Message-ID-Hash"] = add_to_list(listname, msg)
         return msg
 
     def do_tag_post(self, data):
@@ -183,6 +184,18 @@ class ThreadTestCase(TestCase):
         self.assertEqual(response.status_code, 200)
         return json.loads(response.content.decode(response.charset))
 
+    def do_tag_suggest(self, mlist_fqdn, threadid):
+        url = reverse('hk_suggest_tags', args=[mlist_fqdn, threadid])
+        return self.client.get(url)
+
+    def test_suggest_tag(self):
+        self.do_tag_post({"tag": "testtag", "action": "add"})
+
+        msg2 = self._make_msg(msgid="msgid2", listname="list2@example.com")
+        threadid = msg2["Message-ID-Hash"]
+        response = self.do_tag_suggest("list2@example.com", threadid)
+        self.assertEqual(response.status_code, 200)
+
     def test_add_tag(self):
         result = self.do_tag_post({"tag": "testtag", "action": "add"})
         self.assertEqual(result["tags"], ["testtag"])
@@ -293,6 +306,22 @@ class ThreadTestCase(TestCase):
         self.assertTrue("reply-mailto" in link["class"])
         check_mailto(link)
 
+    @override_settings(HYPERKITTY_ALLOW_WEB_POSTING=False)
+    def test_reply_button_when_disabled_posting(self):
+        url = reverse('hk_thread', args=["list@example.com", self.threadid])
+        # Authenticated request
+        response = self.client.get(url)
+        soup = BeautifulSoup(response.content, "html.parser")
+        self.assertEqual(len(soup.find_all("a", class_="reply-mailto")), 0)
+        self.assertIsNone(soup.find("a", class_="reply"))
+
+        # Anonymous request.
+        self.client.logout()
+        response = self.client.get(url)
+        soup = BeautifulSoup(response.content, "html.parser")
+        self.assertEqual(len(soup.find_all("a", class_="reply-mailto")), 0)
+        self.assertIsNone(soup.find("a", class_="reply"))
+
     def test_subject_changed(self):
         # Test the detection of subject change
         self._make_msg("msgid2", {"In-Reply-To": "<msgid>",
@@ -330,19 +359,6 @@ class ThreadTestCase(TestCase):
         self.assertContains(response, '"email-body fixed"', count=1)
         self.assertContains(response, '"email-body "', count=1)
 
-    def test_email_escaped_body(self):
-        msg = EmailMessage()
-        msg["From"] = "Dummy Sender <dummy@example.com>"
-        msg["Subject"] = "Dummy Subject"
-        msg["Date"] = "Mon, 02 Feb 2015 13:00:00 +0300"
-        msg["Message-ID"] = "<msgid2>"
-        msg["In-Reply-To"] = "<msgid>"
-        msg.set_payload("Email address: email@example.com")
-        add_to_list("list@example.com", msg)
-        url = reverse('hk_thread', args=["list@example.com", self.threadid])
-        response = self.client.get(url)
-        self.assertNotContains(response, "email@example.com", status_code=200)
-
     def test_email_in_link_in_body(self):
         msg = EmailMessage()
         msg["From"] = "Dummy Sender <dummy@example.com>"
@@ -355,8 +371,11 @@ class ThreadTestCase(TestCase):
         add_to_list("list@example.com", msg)
         url = reverse('hk_thread', args=["list@example.com", self.threadid])
         response = self.client.get(url)
+        with open('example.html', 'wb') as fd:
+            fd.write(response.content)
         self.assertContains(
-            response, '<a href="{0}" rel="nofollow">{0}</a>'.format(link),
+            response,
+            '<a target="_blank" href="{}">http://example.com/list/email@example.com/message</a>'.format(link),  # noqa: E501
             status_code=200)
 
     def test_email_escaped_sender(self):
@@ -416,3 +435,29 @@ class ThreadTestCase(TestCase):
             resp["replies_html"].count('div class="email unread">'), 3)
         self.assertFalse(resp["more_pending"])
         self.assertIsNone(resp["next_offset"])
+
+    def test_replies_have_reply_button(self):
+        msg = self._make_msg('id1', {'Subject': 'Starting email'})
+        threadid = msg.get('Message-ID-Hash')
+        self._make_msg('id2', {
+            'In-Reply-To': '<id1>', 'Subject': 'Re: Starting email'
+        })
+        url = reverse('hk_thread_replies', args=('list@example.com', threadid))
+        response = self.client.get(url)
+        resp = json.loads(response.content.decode(response.charset))
+        self.assertEqual(
+            resp['replies_html'].count('div class="email unread"'), 1)
+        self.assertEqual(
+            resp['replies_html'].count('a class="reply"'), 1)
+
+        # When the web posting is disabled, the reply button shouldn't show up.
+        with override_settings(HYPERKITTY_ALLOW_WEB_POSTING=False):
+            response = self.client.get(url)
+            resp = json.loads(
+                response.content.decode(response.charset))
+            self.assertEqual(
+                resp['replies_html'].count(
+                    'div class="email unread"'), 1)
+            self.assertEqual(
+                resp['replies_html'].count(
+                    'a class="reply"'), 0)
